﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

//Import extensions
using SharpGIS.MouseExtensions;

namespace MouseExtensionTests
{
	public partial class Page : UserControl
	{
		public Page()
		{
			InitializeComponent();
			dblclick.AttachDoubleClick(OnClick);
			click.AttachClick(OnClick);
			rightclick.AttachRightClick(OnRightClick);
			hover.AttachHover(OnHover);
			ctrlclick.AttachKeyClick(OnClick, ModifierKeys.Control);
			shiftclick.AttachKeyClick(OnClick, ModifierKeys.Shift);
			drag.AttachDrag(OnDrag);
			wheel.AttachWheel(OnWheel);
		}

		private void OnClick(object sender, MouseButtonEventArgs args)
		{
			PositionContextMenu(args.GetPosition(contextMenu.Parent as UIElement), true);
		}

		private void OnHover(object sender, MouseEventArgs args)
		{
			PositionContextMenu(args.GetPosition(contextMenu.Parent as UIElement), true);
		}

		private void OnWheel(object sender, MouseWheelEventArgs args)
		{
			Point p = args.GetPosition(wheel);
			wheelStatus.Text = string.Format("Delta: {0}. Location: {1},{2}",
				args.Delta, p.X, p.Y);

			PositionContextMenu(args.GetPosition(contextMenu.Parent as UIElement), true);
			args.Handled = true; //Prevents page from scrolling
		}

		private void OnRightClick(object sender, SharpGIS.MouseExtensions.RightMouseButtonEventArgs args)
		{
			PositionContextMenu(args.GetPosition(contextMenu.Parent as UIElement), true);
			args.Handled = true;
		}

		private void PositionContextMenu(Point p, bool useTransition)
		{
			if(useTransition)
				contextMenu.IsOpen = false;
			contextMenu.HorizontalOffset = p.X;
			contextMenu.VerticalOffset = p.Y;
			contextMenu.IsOpen = true;
		}

		private void contextMenu_Opened(object sender, EventArgs e)
		{
			ShowPopup.Begin();
		}

		private void contextMenu_Closed(object sender, EventArgs e)
		{
			HidePopup.Begin();
		}

		private void OnDrag(object sender, SharpGIS.MouseExtensions.Drag.MouseDragEventArgs args)
		{
			Point pStart = args.DragStart.GetPosition(drag);
			Point pNow;
			if (args.CurrentDrag != null) //still dragging
				pNow = args.CurrentDrag.GetPosition(drag);
			else if (args.DragEnd != null) //dragging ended
				pNow = args.DragEnd.GetPosition(drag);
			else pNow = new Point(double.NaN, double.NaN); //user dragged outside the plugin
			dragStatus.Text = string.Format("{0} {1},{2} -> {3},{4}",
				args.CurrentDrag == null ? "drag ended" : "dragging",
				pStart.X, pStart.Y, pNow.X, pNow.Y);

			if (!double.IsNaN(pNow.X))
			{
				Point menuPos = drag.TransformToVisual(contextMenu.Parent as UIElement).Transform(pNow);
				menuPos.X += 10;
				menuPos.Y += 10;
				PositionContextMenu(menuPos, false);
			}
		}
	}
}
