﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;

namespace SharpGIS.MouseExtensions
{
	public static class Click
	{
		private static readonly DependencyProperty ClickHandlersProperty = DependencyProperty.RegisterAttached("ClickHandlers", typeof(IList<MouseButtonEventHandler>), typeof(Click), null);
		private static readonly DependencyProperty ClickPositionProperty = DependencyProperty.RegisterAttached("ClickPosition", typeof(Point), typeof(Click), null);

		public static void AttachClick(this UIElement element, MouseButtonEventHandler handler)
		{
			IList<MouseButtonEventHandler> handlers = element.GetValue(ClickHandlersProperty) as IList<MouseButtonEventHandler>;
			if (handlers == null)
			{
				element.MouseLeftButtonDown += element_MouseLeftButtonDown;
				element.MouseLeftButtonUp += element_MouseLeftButtonUp;
				element.MouseMove += element_MouseMove;
				handlers = new List<MouseButtonEventHandler>();
				element.SetValue(ClickHandlersProperty, handlers);
			}
			handlers.Add(handler);
			
		}

		public static void DetachClick(this UIElement element, MouseButtonEventHandler handler)
		{
			IList<MouseButtonEventHandler> handlers = element.GetValue(ClickHandlersProperty) as IList<MouseButtonEventHandler>;
			if (handlers != null)
			{
				if (handlers.Contains(handler))
				{
					handlers.Remove(handler);
					if (handlers.Count == 0)
					{
						element.MouseLeftButtonDown -= element_MouseLeftButtonDown;
						element.MouseLeftButtonUp -= element_MouseLeftButtonUp;
						element.MouseMove -= element_MouseMove;
						element.ClearValue(ClickHandlersProperty);
					}
				}
			}
		}

		private static void element_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			UIElement element = sender as UIElement;
			element.SetValue(ClickPositionProperty, e.GetPosition(element));
		}

		private static void element_MouseMove(object sender, MouseEventArgs e)
		{
			(sender as UIElement).ClearValue(ClickPositionProperty);
		}

		private static void element_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
		{
			UIElement element = sender as UIElement;
			object o = element.GetValue(ClickPositionProperty);
			if (!(o is Point)) return;
			Point pUp = (Point)o;
			Point pDown = e.GetPosition(element);
			if (pUp.X == pDown.X && pUp.Y == pDown.Y)
			{
				IList<MouseButtonEventHandler> handlers = element.GetValue(ClickHandlersProperty) as IList<MouseButtonEventHandler>;
				if(handlers == null) return;
				foreach (MouseButtonEventHandler handler in handlers)
					handler(element, e);
			}
		}
	}
}
