﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;

namespace SharpGIS.MouseExtensions
{
	public static class Drag
	{
		public class MouseDragEventArgs : EventArgs
		{
			public MouseButtonEventArgs DragStart { get; private set; }
			public MouseEventArgs CurrentDrag { get; private set; }
			public MouseButtonEventArgs DragEnd { get; private set; }
			internal MouseDragEventArgs(MouseButtonEventArgs start, MouseButtonEventArgs end, MouseEventArgs drag)
			{
				DragStart = start;
				DragEnd = end;
				CurrentDrag = drag;
			}
		}

		private static UIElement currentElement;
		private static MouseButtonEventArgs start;
		private static readonly DependencyProperty DragHandlersProperty = DependencyProperty.RegisterAttached("DragClickHandlers", typeof(IList<EventHandler<MouseDragEventArgs>>), typeof(Drag), null);
		
		public static void AttachDrag(this UIElement element, EventHandler<MouseDragEventArgs> handler)
		{
			IList<EventHandler<MouseDragEventArgs>> handlers = element.GetValue(DragHandlersProperty) as IList<EventHandler<MouseDragEventArgs>>;
			if (handlers == null)
			{
				element.MouseLeftButtonDown += element_MouseLeftButtonDown;
				handlers = new List<EventHandler<MouseDragEventArgs>>();
				element.SetValue(DragHandlersProperty, handlers);
			}
			handlers.Add(handler);
		}

		public static void DetachDrag(this UIElement element, EventHandler<MouseDragEventArgs> handler)
		{
			IList<EventHandler<MouseDragEventArgs>> handlers = element.GetValue(DragHandlersProperty) as IList<EventHandler<MouseDragEventArgs>>;
			if (handlers != null)
			{
				if (handlers.Contains(handler))
				{
					handlers.Remove(handler);
					if (handlers.Count == 0)
					{
						element.ClearValue(DragHandlersProperty);
						element.MouseLeftButtonDown -= element_MouseLeftButtonDown;

					}
				}
			}
		}
		private static void element_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			currentElement = sender as UIElement;
			Point position = e.GetPosition(currentElement);
			Application.Current.RootVisual.MouseMove += RootVisual_MouseMove;
			Application.Current.RootVisual.MouseLeftButtonUp += RootVisual_MouseLeftButtonUp;
			Application.Current.RootVisual.MouseLeave += new MouseEventHandler(RootVisual_MouseLeave);
			start = e;
		}

		static void RootVisual_MouseLeave(object sender, MouseEventArgs e)
		{
			RaiseDragged(null);
		}

		private static void RootVisual_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
		{
			RaiseDragged(e);
		}

		private static void RaiseDragged(MouseButtonEventArgs e)
		{
			Application.Current.RootVisual.MouseMove -= RootVisual_MouseMove;
			Application.Current.RootVisual.MouseLeftButtonUp -= RootVisual_MouseLeftButtonUp;
			if (currentElement == null) return;
			IList<EventHandler<MouseDragEventArgs>> handlers = currentElement.GetValue(DragHandlersProperty) as IList<EventHandler<MouseDragEventArgs>>;
			if (handlers != null)
			{
				MouseDragEventArgs args = new MouseDragEventArgs(start, e, null);
				foreach (EventHandler<MouseDragEventArgs> handler in handlers)
					handler(currentElement, args);
			}
			currentElement = null;
		}

		private static void RootVisual_MouseMove(object sender, MouseEventArgs e)
		{
			IList<EventHandler<MouseDragEventArgs>> handlers = currentElement.GetValue(DragHandlersProperty) as IList<EventHandler<MouseDragEventArgs>>;
			if (handlers != null)
			{
				MouseDragEventArgs args = new MouseDragEventArgs(start, null, e);
				foreach (EventHandler<MouseDragEventArgs> handler in handlers)
					handler(currentElement, args);
			}
		}
	}
}
