﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;
using System.Windows.Threading;

namespace SharpGIS.MouseExtensions
{
	public static class Hover
	{
		private const double hoverTime = 500;
		private static UIElement currentElement;
		private static MouseEventArgs currentArgs;
		private static DispatcherTimer timer;
		private static DispatcherTimer Timer
		{
			get
			{
				if (timer == null)
				{
					timer = new DispatcherTimer() { Interval = TimeSpan.FromMilliseconds(hoverTime) };
					timer.Tick += timer_Tick;
				}
				return timer;
			}
		}

		private static readonly DependencyProperty HoverHandlersProperty = DependencyProperty.RegisterAttached("HoverHandlers", typeof(IList<MouseEventHandler>), typeof(Hover), null);
		
		public static void AttachHover(this UIElement element, MouseEventHandler handler)
		{
			IList<MouseEventHandler> handlers = element.GetValue(HoverHandlersProperty) as IList<MouseEventHandler>;
			if (handlers == null)
			{
				element.MouseLeave += element_MouseLeave;
				element.MouseMove += element_MouseMove;
				handlers = new List<MouseEventHandler>();
				element.SetValue(HoverHandlersProperty, handlers);
			}
			handlers.Add(handler);
		}

		public static void DetachHover(this UIElement element, MouseEventHandler handler)
		{
			IList<MouseEventHandler> handlers = element.GetValue(HoverHandlersProperty) as IList<MouseEventHandler>;
			if (handlers != null)
			{
				if (handlers.Contains(handler))
				{
					handlers.Remove(handler);
					if (handlers.Count == 0)
					{
						element.MouseLeave -= element_MouseLeave;
						element.MouseMove -= element_MouseMove;
						element.ClearValue(HoverHandlersProperty);
					}
				}
			}
		}

		private static void element_MouseLeave(object sender, MouseEventArgs e)
		{
			Timer.Stop();
			currentElement = null;
		}
		
		private static void element_MouseMove(object sender, MouseEventArgs e)
		{
			Timer.Start();
			currentElement = sender as UIElement;
			currentArgs = e;
		}

		private static void timer_Tick(object sender, EventArgs e)
		{
			Timer.Stop();
			if (currentElement != null)
			{
				currentElement.GetValue(HoverHandlersProperty);
				IList<MouseEventHandler> handlers = currentElement.GetValue(HoverHandlersProperty) as IList<MouseEventHandler>;
				if (handlers == null) return;
				foreach (MouseEventHandler handler in handlers)
					handler(currentElement, currentArgs);
				currentArgs = null;
			}
		}
	}
}
