﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;

namespace SharpGIS.MouseExtensions
{
	public static class KeyClick
	{
		private static readonly DependencyProperty KeyClickHandlersProperty =
			DependencyProperty.RegisterAttached("KeyClickHandlers", typeof(IDictionary<ModifierKeys, IList<MouseButtonEventHandler>>), typeof(KeyClick), null);

		/// <summary>
		/// Adds event listener for click on the element with the specified key modifier.
		/// </summary>
		/// <param name="element">The element.</param>
		/// <param name="handler">The handler.</param>
		/// <param name="key">The key.</param>
		public static void AttachKeyClick(this UIElement element, MouseButtonEventHandler handler, ModifierKeys key)
		{
			IDictionary<ModifierKeys, IList<MouseButtonEventHandler>> handlers = element.GetValue(KeyClickHandlersProperty) as IDictionary<ModifierKeys, IList<MouseButtonEventHandler>>;
			if (handlers == null)
			{
				handlers = new Dictionary<ModifierKeys, IList<MouseButtonEventHandler>>();
				element.SetValue(KeyClickHandlersProperty, handlers);
				element.AttachClick(onClick);
			}
			if (!handlers.ContainsKey(key))
			{
				handlers.Add(key, new List<MouseButtonEventHandler>());
			}
			handlers[key].Add(handler);
		}

		public static void DetachKeyClick(this UIElement element, MouseButtonEventHandler handler, ModifierKeys key)
		{
			IDictionary<ModifierKeys, IList<MouseButtonEventHandler>> handlers = element.GetValue(KeyClickHandlersProperty) as IDictionary<ModifierKeys, IList<MouseButtonEventHandler>>;
			if (handlers != null)
			{
				if (handlers.ContainsKey(key))
				{
					if (handlers[key].Contains(handler))
						handlers[key].Remove(handler);
					if (handlers[key].Count == 0)
						handlers.Remove(key);
					if (handlers.Count == 0)
					{
						element.ClearValue(KeyClickHandlersProperty);
						element.DetachClick(onClick);
					}
				}
			}
		}

		private static void onClick(object sender, MouseButtonEventArgs args)
		{
			UIElement elm = sender as UIElement;
			IDictionary<ModifierKeys, IList<MouseButtonEventHandler>> handlers =
				elm.GetValue(KeyClickHandlersProperty) as IDictionary<ModifierKeys, IList<MouseButtonEventHandler>>;
			foreach (ModifierKeys key in handlers.Keys)
			{
				if (Keyboard.Modifiers == key)
				{
					foreach (MouseButtonEventHandler handler in handlers[key])
						handler(elm, args);
				}
			}
		}
	}
}
